#include "mpi.h"
#include <assert.h>
#include <stdlib.h>
#include <time.h>   

// Read from the console a number n which is then communicated to all
// proceses. Each process chooses a random number between 1 and n
// and communicates this to the others. Each process calculates the
// maximum of all chosen numbers
int main(int argc, char* argv[])
{
  int n, my_mpi_rank, n_mpi_procs;

  MPI_Init (&argc, &argv);

  MPI_Comm_rank (MPI_COMM_WORLD, &my_mpi_rank);
  MPI_Comm_size (MPI_COMM_WORLD, &n_mpi_procs);

  if (my_mpi_rank == 0)
  {
    printf ("Please, introduce an integer number:\n");
    scanf ("%d", &n);
    assert (n > 0);
  }

  // broadcast n to all ranks
  MPI_Bcast(&n, 1, MPI_INT, 0, MPI_COMM_WORLD);

  srand (my_mpi_rank + 1);
  int my_bid = rand() % n  + 1;
  printf ("rank %d: my bid is %d\n", my_mpi_rank, my_bid);
  
  int max = my_bid;

  for (int rank = 0; rank < n_mpi_procs; rank++)
  {
    if (rank == my_mpi_rank)
      continue;

    MPI_Send
      (&my_bid, 1, MPI_INT, rank, 100, MPI_COMM_WORLD);

    printf ("Sent %d to rank %d\n", my_bid, rank);
  }
  
  for (int rank = 0; rank < n_mpi_procs; rank++)
  {
    if (rank == my_mpi_rank)
      continue;

    int bid;
    MPI_Status status;
    
    MPI_Recv
      ( &bid, 1, MPI_INT
      , rank, 100, MPI_COMM_WORLD
      , &status
      );

    printf ("Received %d from rank %d\n", bid, rank);
    if (bid >max) max = bid;
  }
  
  if (my_mpi_rank == 0)
  {
    printf ("The maximum bid is %d\n", max);
  }

  MPI_Finalize();

  return 0;
}
